/**
 Use the Win32 API routine "MapAndLoad" to extract 
 information about an EXE file.
 
 Orion Sky Lawlor, olawlor@acm.org, 2005/3/31 (Public Domain)
 Based on CS 321 Project1 Code by Tim Kvitek
*/
#include "windows.h"
#include "imagehlp.h" /* Windows header file: PE dump routines. */
#pragma comment(lib,"imagehlp.lib") /* Link with image routine library */

#include <iostream>
#include <string>
using namespace std;

void dumpEXE(const char *exeFile);

int main(int argc,char *argv[])
{
	if (argc>1) {dumpEXE(argv[1]); return 0;}
	cout << "Usage: dumpEXE <exe file name>" << endl;
	return 0;
}

/// Print this integer size field.
void printSize(const char *what,int bytes) {
	cout<<"Size of "<<what<<": "<<
		dec<<bytes<<" bytes "<<
		"(0x"<<hex<<bytes<<dec<<")\n";
}

/// Print this address.
void printAddr(const char *what,void *ptr) {
	cout<<"Address of "<<what<<": 0x"<<ptr<<"\n";
}

/// Print a selection of information about this EXE file
///  to the screen.
void dumpEXE(const char *exeFile)
{
	LOADED_IMAGE li;
	bool ok = MapAndLoad((PSTR)exeFile,NULL,&li, 0, 1);
	if (!ok) {cout<<"Error opening file "<<exeFile<<"!\n";exit(1);}
	
	cout<<"Image name: '"<<exeFile<<"'\n";
	printSize("image",li.SizeOfImage);
	printAddr("loaded image",(void *)li.MappedAddress);
	
	IMAGE_OPTIONAL_HEADER *h=&li.FileHeader->OptionalHeader;
	printSize("code",h->SizeOfCode);
	printSize("initialized data",h->SizeOfInitializedData);
	printSize("uninitialized data",h->SizeOfUninitializedData);
	printAddr("code",(void *)h->BaseOfCode);
	printAddr("code entry point",(void *)h->AddressOfEntryPoint);
	printAddr("data",(void *)h->BaseOfData);
	printAddr("image",(void *)h->ImageBase);
	cout<<"Linker version: "<<(int)h->MajorLinkerVersion<<"."<<(int)h->MinorLinkerVersion<<"\n";
}
